
// *** UTILITY FUNCTIONS *** //

Array.prototype.isEmpty = function(){
	
	return( this.length === 0 );
	
}


function Fader( length ){
	
	var fader       = this,
		timeElapsed = 0,
		timeTotal   = length * fps,
		isFadingOut = false;
	
	this.complete = false;
	this.opacity = 0;
	
	this.fade = function(){
		
		tick();
		transition();
		shiftOpacity();
		
	}
	
	function tick(){
		
		timeElapsed = ++timeElapsed % timeTotal;
		
	}
	
	function transition(){
		
		if( !timeElapsed ){
			
			if( isFadingOut ) fader.complete    = true;
			else              isFadingOut = true;
			
		}
		
	}
	
	function shiftOpacity(){
		
		var fadePercent = timeElapsed / timeTotal;
		
		if( isFadingOut ) fader.opacity = 1 - fadePercent;
		else              fader.opacity = fadePercent;
		
	}
	
}


function SineCycle( steps, amplitude ){
	
	// *** INIT *** //
	
	var sineCycle = this,	
		step      = 0,
		amplitude = amplitude || 1;
	
	this.value    = 0;
	this.vertex   = false;
	this.complete = false;
	
	// *** METHODS *** //
	
	this.update = function(){
		
		computeValue();
		testForVertex();
		testForCompletion();
		incrementStep();
		
		return this;
		
	}
	
	this.reset = function(){
		
		step          = 0;
		this.value    = 0;
		this.vertex   = false;
		this.complete = false;
		
		
	};
	
	function computeValue(){
		
		sineCycle.value = amplitude * Math.sin( step / steps * Math.PI );
		
	}
	
	function testForVertex(){
		
		this.vertex = false;
		
		if( step == steps / 2 ) sineCycle.vertex = true;
		
	}
	
	function testForCompletion(){
		
		if( step == steps ) sineCycle.complete = true;
		
	}
	
	function incrementStep(){
		
		step++;
		
	}
	
}


function Timer( length, callback, args ){
	
	
	// *** INIT *** //
	
	var time,
		timeElapsed = 0;
		
		setTime();
	
	
	// *** METHODS *** //
	
	this.tick = function(){
		
		timeElapsed = ++timeElapsed % time;
		
		if( !timeElapsed ){ 
			callback( args );
			setTime();
		}
		
	}
	
	
	this.reset = function(){
		
		timeElapsed = 0;
		setTime();
		
	}

	
	function setTime(){
		
		if( typeof length === 'number' )       time = length;
		else if( typeof length === 'function') time = length();
		else if( Array.isArray( length ) )     time = randomInt( length[0], length[1] );
		
	}
	
}


function parseFilename( src ){
		
	var filename = '',
		firstChar = src.lastIndexOf("/") + 1,
		length = src.length;
	
	for( var char = firstChar; char < length; char++ ){
		if( src[ char ] === "." ) break;
		filename += src[ char ];
	}
	
	return filename;

}


function isNull( variable ){
	
	return variable === null;
	
}


function limit( value, max ){
	
	return Math.min( value, max );
	
}


// have not been refactored

function within(rect){
    return (this.x > rect.x && this.x < rect.x + rect.w && this.y > rect.y && this.y < rect.y + rect.h);
}

function isDefined(v){
    return (typeof v !== 'undefined');
}

function plusOrMinus(val){
    if(randomInt(0,1))
        return val;
    return -val;
}

function randomInt( min, max ){
    return Math.floor( Math.random() * ( max - min + 1 )) + min;
}

function randomFloat( min, max ){
    return Math.random() * ( max - min ) + min;
}

function toDegrees( angle ){
    return angle * 180 / Math.PI;
}

function toRadians( angle ){
    return angle / 180 * Math.PI;
}

function getDistance(x1,y1,x2,y2){
    return Math.sqrt(Math.pow(x1-x2,2) + Math.pow(y1-y2,2));
}

//Code Modified From: https://gist.github.com/conorbuck/2606166
function getAngle( x1, y1, x2, y2 ){
    
	if( x1 === x2 && y1 === y2 ) return null;
	
	return Math.atan2( y2 - y1, x2 - x1 );

}

function drawCircle(c,x,y,size,color){
    c.beginPath();
    c.arc(x, y, size, 0, 2 * Math.PI);
    c.fillStyle = color || '#FFF';
    c.fill();
}

function drawLine(x1,y1,x2,y2,w,color){
    c.beginPath();
    c.lineWidth = w;
    c.strokeStyle = color;
    c.moveTo(x1,y1);
    c.lineTo(x2,y2);
    c.stroke();
}

function drawLightning(x1,y1,x2,y2){
    var x = x1,
        y = y1,
        nx,
        ny,
        distance,
        direction;
    
    drawCircle(c,x,y,3,'aqua');
    while(true){
        distance = randomInt(1,5);
        direction = getAngle(x,y,x2,y2) + randomFloat(-Math.PI/6,Math.PI/6);
        nx = x + distance * Math.cos(direction);
        ny = y + distance * Math.sin(direction);
        if(getDistance(nx ,ny ,x2 ,y2) > 5){
            drawLine(x, y, nx, ny, "5", "aqua");
            drawLine(x,y,nx,ny,'1','blue');
            x = nx;
            y = ny;
        }else{
            drawCircle(c,x,y,3,'aqua');
            drawLine(x, y, nx, ny, "5", "aqua");
            drawLine(x,y,nx,ny,'1','blue');
            break;
        }
    }
}
